MODULE CONJUGATE
IMPLICIT NONE

	INTEGER :: ncom
	REAL(8), DIMENSION(:), POINTER :: pcom,xicom

    CONTAINS
    
	SUBROUTINE powell(np,p,xi,ftol,iter,fret,func)
	IMPLICIT NONE
	INTEGER, INTENT(IN) :: np
	REAL(8), INTENT(INOUT) :: p(np)
	REAL(8), INTENT(INOUT) :: xi(np,np)
	INTEGER, INTENT(OUT) :: iter
	REAL(8), INTENT(IN) :: ftol
	REAL(8), INTENT(OUT) :: fret
	INTERFACE
		REAL(8) FUNCTION func(np,p)
		IMPLICIT NONE
		INTEGER, INTENT(IN) :: np
		REAL(8), INTENT(IN) :: p(np)
		END FUNCTION func
	END INTERFACE
	INTEGER, PARAMETER :: ITMAX=200
	REAL(8), PARAMETER :: TINY=1.0d-25
	INTEGER :: i,ibig
	REAL(8) :: del,fp,fptt,t
	REAL(8) :: pt(np),ptt(np),xit(np)
   
	fret=func(np,p)
	pt = p
	iter = 0
	do
		iter = iter+1
		fp = fret
		ibig = 0
		del = 0.0d0
		do i = 1, np
			xit = xi(:,i)
			fptt = fret
			call linmin(np,p,xit,fret,func)
			if (fptt-fret > del) then
				del = fptt-fret
				ibig = i
			end if
		end do
		if (2.0d0*(fp-fret) <= ftol*(abs(fp)+abs(fret))+TINY) RETURN
		if (iter == ITMAX) write(*,*)'powell exceeding maximum iterations'
		ptt = 2.0d0*p - pt
		xit = p - pt
		pt = p
		fptt = func(np,ptt)
		if (fptt >= fp) cycle
		t=2.0d0*(fp-2.0d0*fret+fptt)*(fp-fret-del)**2-del*(fp-fptt)**2
		if (t >= 0.0) cycle
		call linmin(np,p,xit,fret,func)
		xi(:,ibig)=xi(:,np)
		xi(:,np)=xit
	end do
	END SUBROUTINE powell

	SUBROUTINE mnbrak(np,p,ax,bx,cx,fa,fb,fc,func)
	IMPLICIT NONE
	INTEGER, INTENT(IN) :: np
	REAL(8), INTENT(IN) :: p(np)
	REAL(8), INTENT(INOUT) :: ax,bx
	REAL(8), INTENT(OUT) :: cx,fa,fb,fc
    INTERFACE
		REAL(8) FUNCTION func(np,p)
		IMPLICIT NONE
		INTEGER, INTENT(IN) :: np
		REAL(8), INTENT(IN) :: p(np)
		END FUNCTION func
	END INTERFACE
	REAL(8), PARAMETER :: GOLD=1.618034d0
	REAL(8), PARAMETER :: GLIMIT=100.0d0
	REAL(8), PARAMETER :: TINY=1.0d-20

	REAL(8) :: fu,q,r,u,ulim,dum
	fa = f1dim(np,p,ax,func)
	fb = f1dim(np,p,bx,func)
	if (fb > fa) then
	    !swap(ax,bx)
	    dum = ax
	    ax = bx 
	    bx = dum 
	    !swap(fa,fb)
	    dum = fa
	    fa = fb 
	    fb = dum 
	end if
	cx=bx+GOLD*(bx-ax)
	fc = f1dim(np,p,cx,func)
	do
		if (fb < fc) RETURN
		r = (bx-ax)*(fb-fc)
		q = (bx-cx)*(fb-fa)
		u = bx-((bx-cx)*q-(bx-ax)*r)/(2.0d0*sign(max(abs(q-r),TINY),q-r))
		ulim = bx+GLIMIT*(cx-bx)
		if ((bx-u)*(u-cx) > 0.0d0) then
			fu = f1dim(np,p,u,func)
			if (fu < fc) then
				ax = bx
				fa = fb
				bx = u
				fb = fu
				RETURN
			else if (fu > fb) then
				cx = u
				fc = fu
				RETURN
			end if
			u = cx + GOLD*(cx-bx)
			fu = f1dim(np,p,u,func)
		else if ((cx-u)*(u-ulim) > 0.0d0) then
			fu = f1dim(np,p,u,func)
			if (fu < fc) then
				bx = cx
				cx = u
				u = cx + GOLD*(cx-bx)
				call shft(fb,fc,fu,f1dim(np,p,u,func))
			end if
		else if ((u-ulim)*(ulim-cx) >= 0.0d0) then
			u = ulim
			fu = f1dim(np,p,u,func)
		else
			u = cx + GOLD*(cx-bx)
			fu = f1dim(np,p,u,func)
		end if
		call shft(ax,bx,cx,u)
		call shft(fa,fb,fc,fu)
	end do
	END SUBROUTINE mnbrak

	REAL(8) FUNCTION brent(np,pnew,ax,bx,cx,tol,xmin,func)
	IMPLICIT NONE
	INTEGER, INTENT(IN) :: np
	REAL(8), INTENT(IN) :: pnew(np)
	REAL(8), INTENT(IN) :: ax,bx,cx,tol
	REAL(8), INTENT(OUT) :: xmin
    INTERFACE
		REAL(8) FUNCTION func(np,p)
		IMPLICIT NONE
		INTEGER, INTENT(IN) :: np
		REAL(8), INTENT(IN) :: p(np)
		END FUNCTION func
	END INTERFACE
	INTEGER, PARAMETER :: ITMAX=100
	REAL(8), PARAMETER :: CGOLD=0.3819660d0
	REAL(8), PARAMETER :: ZEPS=1.0d-3*epsilon(ax)
	INTEGER :: iter
	REAL(8) :: a,b,d,e,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm
	a = min(ax,cx)
	b = max(ax,cx)
	v = bx
	w = v
	x = v
	e = 0.0d0
	fx = f1dim(np,pnew,x,func)
	fv = fx
	fw = fx
	do iter = 1, ITMAX
		xm = 0.5d0*(a+b)
		tol1 = tol*abs(x)+ZEPS
		tol2 = 2.0d0*tol1
		if (abs(x-xm) <= (tol2-0.5d0*(b-a))) then
			xmin = x
			brent = fx
			RETURN
		end if
		if (abs(e) > tol1) then
			r = (x-w)*(fx-fv)
			q = (x-v)*(fx-fw)
			p = (x-v)*q-(x-w)*r
			q = 2.0d0*(q-r)
			if (q > 0.0d0) p=-p
			q = abs(q)
			etemp = e
			e = d
			if (abs(p) >= abs(0.5d0*q*etemp) .or. p <= q*(a-x) .or. p >= q*(b-x)) then
				e = merge(a-x,b-x, x >= xm )
				d = CGOLD*e
			else
				d = p/q
				u = x+d
				if (u-a < tol2 .or. b-u < tol2) d=sign(tol1,xm-x)
			end if
		else
			e = merge(a-x,b-x, x >= xm )
			d = CGOLD*e
		end if
		u = merge(x+d,x+sign(tol1,d), abs(d) >= tol1 )
		fu = f1dim(np,pnew,u,func)
		if (fu <= fx) then
			if (u >= x) then
				a = x
			else
				b = x
			end if
			call shft(v,w,x,u)
			call shft(fv,fw,fx,fu)
		else
			if (u < x) then
				a = u
			else
				b = u
			end if
			if (fu <= fw .or. w == x) then
				v = w
				fv = fw
				w = u
				fw = fu
			else if (fu <= fv .or. v == x .or. v == w) then
				v = u
				fv = fu
			end if
		end if
	end do
	write(*,*)'brent: exceed maximum iterations'
	END FUNCTION brent

	SUBROUTINE shft(a,b,c,d)
	REAL(8), INTENT(OUT) :: a
	REAL(8), INTENT(INOUT) :: b,c
	REAL(8), INTENT(IN) :: d
	a=b
	b=c
	c=d
	END SUBROUTINE shft

	SUBROUTINE linmin(np,p,xi,fret,func)
	IMPLICIT NONE
	INTEGER, INTENT(IN) :: np
	REAL(8), INTENT(OUT) :: fret
	REAL(8), TARGET, INTENT(INOUT) :: p(np)
	REAL(8), TARGET, INTENT(INOUT) :: xi(np)
	INTERFACE
		REAL(8) FUNCTION func(np,p)
		IMPLICIT NONE
		INTEGER, INTENT(IN) :: np
		REAL(8), INTENT(IN) :: p(np)
		END FUNCTION func
	END INTERFACE
	REAL(8), PARAMETER :: TOL=1.0d-4
	REAL(8) :: ax,bx,fa,fb,fx,xmin,xx
	ncom = np
	pcom=>p     !pcom points to p, so pcom takes the same values as p. If p changes, so does pcom automatically.
	xicom=>xi   !xicom points to xi, so xicom takes the same values as xi. If xi changes, so does xicom automatically. 
	ax=0.0d0
	xx=1.0d0
	call mnbrak(np,p,ax,xx,bx,fa,fx,fb,func)
	fret=brent(np,p,ax,xx,bx,TOL,xmin,func)
	xi=xmin*xi
	p=p+xi
	END SUBROUTINE linmin

	REAL(8) FUNCTION f1dim(np,p,x,func)   !This is the one dimension function that Powell is minimizing f(p+x*xi)
	IMPLICIT NONE
    INTEGER, INTENT(IN) :: np
    REAL(8), INTENT(IN) :: p(np)
	REAL(8), INTENT(IN) :: x
	INTERFACE
		REAL(8) FUNCTION func(np,p)
		INTEGER, INTENT(IN) :: np
		REAL(8), INTENT(IN) :: p(np)
		END FUNCTION func
	END INTERFACE
	REAL(8), DIMENSION(:), ALLOCATABLE :: xt
	allocate(xt(ncom))
	xt(:)=pcom(:)+x*xicom(:)    !Here is 
	f1dim=func(ncom,xt)
	deallocate(xt)
	END FUNCTION f1dim


END MODULE CONJUGATE