module mod_utilities
use mod_globvar
use mod_matrix
implicit none
contains
real(8) function util(c,l,age,as)
implicit none
real(8), intent(in) :: c, l, age, as   
if ((c>0.0d0).AND.(l>=lmin_func(age)).AND.(l<1.0d0).AND.(as>=dble(amin)).AND.(as<=dble(amax))) then        
util = (((c**(1.0d0-dble(rho_func(age))))*((1.0d0-l)**dble(rho_func(age))))**(1.0d0-gamma))/(1.0d0-gamma)- chi_age(age)
else    
util = -100000000000.0d0        
endif            
end function util
real(8) function util_out(c,l,age,as)
implicit none
real(8), intent(in) :: c, l, age, as    
if ((c>0.0d0).AND.(l>=0.0d0).AND.(l<1.0d0).AND.(as>=dble(amin)).AND.(as<=dble(amax))) then        
util_out = (((c**(1.0d0-dble(rho_func(age))))*((1.0d0-0.0d0)**dble(rho_func(age))))**(1.0d0-gamma))/(1.0d0-gamma) 
else
util_out = -100000000000.0d0
endif        
end function util_out
real(8) function util_beq(as)
implicit none
real(8), intent(in) :: as    
if (as>=dble(amin)) then    
util_beq =  phi_1*(( 1.0d0 + as/dble(phi_2))**(1.0d0-gamma))                 
else    
util_beq = -100000000000.0d0        
endif            
end function util_beq
real(8) function y_func(l,u,z,age)
implicit none
real(8), intent(in) :: l, u, z
integer, intent(in) :: age
if (l>=lmin_func(age)) then    
y_func = w*l*dexp(u+z)*ef_temp(age)                            
else    
y_func = 0.0d0        
end if                                    
end function y_func
real(8) function ben_func(xx)
implicit none
real(8), intent(in) :: xx
real(8) :: y1, y2, theta1, theta2, theta3    
if (model==0.0d0) then
theta1 = 0.90d0
theta2 = 0.32d0    
theta3 = 0.15d0
y1 = 0.21d0*ym  
y2 = 1.19d0*ym  
if (xx<=y1) then    
ben_func = theta1*xx        
else if ((xx>y1).AND.(xx<=y2)) then    
ben_func = theta1*y1 + theta2*(xx - y1)
else if ((xx>y2).AND.(xx<=ymax)) then            
ben_func = theta1*y1 + theta2*(y2- y1) + theta3*(xx- y2)        
else if (xx>ymax) then                                    
ben_func = theta1*y1 + theta2*(y2- y1) + theta3*(ymax- y2)                                    
end if                                
elseif (model==1.0d0) then    
ben_func = max((r_ben/(1.0d0-(1.0d0+dble(r_ben))**(-dble(ntime_ret))))*xx,b_min)      
endif                
end function ben_func
real(8) function tau_ss_func(age)
implicit none
real(8), intent(in) :: age
tau_ss_func = min(max(tau_ss_0 + tau_ss_1*((19.0d0+age)/10.0d0),0.0d0),0.90d0)                                    
end function tau_ss_func
real(8) function Tax_func(y,zeta_1,zeta_2,age)
implicit none
real(8), intent(in) :: y, zeta_1, zeta_2, age            
Tax_func = max(y - (zeta_1 + zeta_2*((19.0d0+age)/10.d0))*(y**(1.0d0-dble(alpha))),0.0d0)         
end function Tax_func
real(8) function trans(cmin,res)
implicit none
real(8), intent(in) :: cmin,res        
trans =  max(cmin-res,0.0d0)                   
end function trans
integer function locate(x,xgrid)
implicit none
double precision, intent(in) :: x, xgrid(:)
integer :: N, ju, jm, jl, jndx	
N  = size(xgrid)
jl = 0
ju = N+1
do while (ju - jl > 1)
jm = (ju + jl)/2
if (x .ge. xgrid(jm)) then
jl = jm
else
ju = jm
end if
end do
if (x .eq. xgrid(1)) then 
jndx = 1
else if (x .eq. xgrid(N)) then
jndx = N - 1
else 
jndx = jl
end if
locate = jndx
end function
double precision function lininterp1(xgrid, ZZ, xx)
implicit none
double precision, intent(in) :: xgrid(:), ZZ(:), xx
double precision :: t
integer :: indx
indx = locate(xx, xgrid)
t = (xx - xgrid(indx))/(xgrid(indx+1) - xgrid(indx))
lininterp1 = (1.0d+0-t)*ZZ(indx) + t*ZZ(indx+1)
end function
double precision function lininterp2(xgrid, ygrid, ZZ, xx, yy)
implicit none
integer :: xndx, yndx
double precision :: tt, uu
double precision, intent(in) :: xx, yy, xgrid(:), ygrid(:), ZZ(:,:)	
xndx = locate(xx, xgrid)
yndx = locate(yy, ygrid)
tt = (xx - xgrid(xndx))/(xgrid(xndx+1) - xgrid(xndx))
uu = (yy - ygrid(yndx))/(ygrid(yndx+1) - ygrid(yndx))
lininterp2 = (1.0d+0-tt)*(1-uu)*ZZ(xndx,yndx) + tt*(1.0d+0-uu)*ZZ(xndx+1,yndx)	&
	+ tt*uu*ZZ(xndx+1,yndx+1) + (1.0d+0-tt)*uu*ZZ(xndx,yndx+1)
end function
subroutine tauchen(N,rho,mu,sigma,X,Xprob)
implicit none	
integer, intent(in) :: N
double precision, intent(in) :: mu, rho, sigma
double precision, intent(out) :: X(N), Xprob(N,N)	
double precision :: xstep, a, b, m
integer :: i, j	
a = (1.0d+0-rho)*mu
b = sigma*sqrt(2.0d+0)
m = 3.0d+0 
X(N) = m*sqrt(sigma*sigma/(1.0d+0-rho*rho))
X(1) = - X(N)
xstep = (X(N) - X(1))/dble(N - 1)
do i = 2, N - 1
X(i) = X(1) + xstep*dble(i - 1)
end do
X = X + a/(1.0d+0-rho)
do i = 1, N
do j = 1, N
if ( j .eq. 1 ) then
Xprob(i,j) = erfc(-(X(1) - a - rho*X(i) + xstep/2.0d+0)/b)
else if ( j .eq. N ) then
Xprob(i,j) = 2.0d+0 - erfc(-(X(N) - a - rho*X(i) - xstep/2.0d+0)/b)
else
Xprob(i,j) = erfc(-(X(j) - a - rho*X(i) + xstep/2.0d+0)/b) - &
                erfc(-(X(j) - a - rho*X(i) - xstep/2.0d+0)/b)
end if
end do
end do
Xprob = Xprob/2.0d+0	
end subroutine
subroutine tauchen_1(N, rho, mu, sigma, X, Xprob)
implicit none	
integer, intent(in) :: N
double precision, intent(in) :: mu, rho, sigma
double precision, intent(out) :: X(N), Xprob(N,N)	
double precision :: xstep, a, b, m
integer :: i, j	
a = (1.0d+0-rho)*mu
b = sigma*sqrt(2.0d+0)
m = 2.62d+0 
X(N)  = m*sqrt(sigma*sigma/(1.0d+0-rho*rho))
X(1)  = - X(N)
xstep = (X(N) - X(1))/dble(N - 1)
do i = 2, N - 1
X(i) = X(1) + xstep*dble(i - 1)
end do
X = X + a/(1.0d+0-rho)
do i = 1, N
do j = 1, N
if ( j .eq. 1 ) then
Xprob(i,j) = erfc(-(X(1) - a - rho*X(i) + xstep/2.0d+0)/b)
else if ( j .eq. N ) then
Xprob(i,j) = 2.0d+0 - erfc(-(X(N) - a - rho*X(i) - xstep/2.0d+0)/b)
else
Xprob(i,j) = erfc(-(X(j) - a - rho*X(i) + xstep/2.0d+0)/b) - &
                erfc(-(X(j) - a - rho*X(i) - xstep/2.0d+0)/b)
end if
end do
end do
Xprob = Xprob/2.0d+0	
end subroutine
end module mod_utilities